using LiftoffSDK.Public;
using System.Collections.Concurrent;

namespace LiftoffWinFormsDemoApp
{
    public partial class FormMain : Form
    {
        private LiftoffAds? m_sdk;
        private ConcurrentDictionary<String, bool> m_placementWithBundles = new ConcurrentDictionary<string, bool>();

        public FormMain()
        {
            InitializeComponent();
        }

        private void FormMain_Load(object sender, EventArgs e)
        {

        }

        private async void buttonStartVungle_Click(object sender, EventArgs e)
        {
            try
            {
                this.buttonStartVungle.Enabled = false;

                LiftoffAds.OnInitializationSuccess -= LiftoffAds_OnInitializationSuccess;
                LiftoffAds.OnInitializationSuccess += LiftoffAds_OnInitializationSuccess;

                LiftoffAds.OnInitializationFailure -= LiftoffAds_OnInitializationFailure;
                LiftoffAds.OnInitializationFailure += LiftoffAds_OnInitializationFailure;

                LiftoffAds.OnAdPlayableChanged -= LiftoffAds_OnAdPlayableChanged;
                LiftoffAds.OnAdPlayableChanged += LiftoffAds_OnAdPlayableChanged;

#pragma warning disable CS8622 // Nullability of reference types in type of parameter doesn't match the target delegate (possibly because of nullability attributes).
                LiftoffAds.AddDiagnosticListener(LiftoffAds_OnDiagnostic);
#pragma warning restore CS8622 // Nullability of reference types in type of parameter doesn't match the target delegate (possibly because of nullability attributes).

                LiftoffAds.OnAdStart -= LiftoffAds_OnAdStart;
                LiftoffAds.OnAdStart += LiftoffAds_OnAdStart;

                LiftoffAds.OnAdEnd -= LiftoffAds_OnAdEnd;
                LiftoffAds.OnAdEnd += LiftoffAds_OnAdEnd;

                LiftoffSdkConfig sdkConfig = new LiftoffSdkConfig
                {
                    ApiEndpoint = new Uri(this.textBoxApiEndpoint.Text.Trim())
                };
                m_sdk = await LiftoffAds.InitializeAsync(this.textBoxAppId.Text.Trim(), sdkConfig);

            }
            catch (Exception ex)
            {
                ShowException(ex);
            }
        }

        private void listBoxPlacements_SelectedValueChanged(object sender, EventArgs e)
        {
            try
            {
                this.buttonLoad.Enabled = (this.listBoxPlacements.SelectedIndex >= 0);

                if (this.listBoxPlacements.SelectedItem == null)
                {
                    return;
                }

                String? selectedPlacement = listBoxPlacements.SelectedItem.ToString();

                if (!String.IsNullOrWhiteSpace(selectedPlacement) && m_placementWithBundles.ContainsKey(selectedPlacement))
                {
                    this.buttonPlay.Enabled = m_placementWithBundles[selectedPlacement];
                }
                else
                {
                    this.buttonPlay.Enabled = false;
                }
            }
            catch (Exception ex)
            {
                ShowException(ex);
            }
        }

        private void buttonLoad_Click(object sender, EventArgs e)
        {
            try
            {
                if (this.listBoxPlacements.SelectedItem == null)
                {
                    return;
                }

                String? placement = listBoxPlacements?.SelectedItem.ToString();
                if (placement != null && m_sdk != null)
                {
                    m_sdk.LoadAd(placement);
                }
            }
            catch (Exception ex)
            {
                ShowException(ex);
            }
        }

        private void buttonPlay_Click(object sender, EventArgs e)
        {
            try
            {
                if (listBoxPlacements.SelectedItem == null)
                {
                    return;
                }

                String? placement = listBoxPlacements?.SelectedItem.ToString();
                AdConfig adConfig = new AdConfig();
                if (placement != null && m_sdk != null)
                {
                    m_sdk.PlayAdAsync(adConfig, placement);
                }
            }
            catch (Exception ex)
            {
                ShowException(ex);
            }
        }

        private void ShowException(Exception ex)
        {
            MessageBox.Show(ex.ToString());
        }

        #region Liftoff Events
        private void LiftoffAds_OnDiagnostic(object? sender, DiagnosticLogEvent e)
        {
            System.Diagnostics.Debug.WriteLine(e.ToString());
        }

        private void LiftoffAds_OnAdEnd(object? sender, AdEndEventArgs e)
        {
            System.Diagnostics.Debug.WriteLine("Ad has ended.");
        }

        private void LiftoffAds_OnAdStart(object? sender, AdStartEventArgs e)
        {
            System.Diagnostics.Debug.WriteLine("Ad has started playing...");
        }

        private void LiftoffAds_OnAdPlayableChanged(object? sender, AdPlayableEventArgs e)
        {
            try
            {
                System.Diagnostics.Debug.WriteLine(
                    String.Format("Liftoff Placement {0} is playable : {1}.", e.Placement, e.AdPlayable));
                m_placementWithBundles[e.Placement] = e.AdPlayable;

                this.buttonPlay.Invoke((MethodInvoker)delegate
                {
                    if (e.Placement.Equals(this.listBoxPlacements.SelectedItem))
                    {
                        this.buttonPlay.Enabled = e.AdPlayable;
                    }
                });
            }
            catch (Exception ex)
            {
                ShowException(ex);
            }
        }

        private void LiftoffAds_OnInitializationFailure(object? sender, InitializationFailureEventArgs e)
        {
            System.Diagnostics.Debug.WriteLine("Liftoff SDK failed initialization.");
        }

        private void LiftoffAds_OnInitializationSuccess(object? sender, InitializationSuccessEventArgs e)
        {
            System.Diagnostics.Debug.WriteLine("Liftoff SDK initialized successfully!");
            try
            {
                this.listBoxPlacements.Invoke((MethodInvoker)delegate
                {
                    foreach (var placementInfo in e.Placements)
                    {
                        this.listBoxPlacements.Items.Add(placementInfo.Name);
                    }
                });
            }
            catch (Exception ex)
            {
                ShowException(ex);
            }
        }
        #endregion
    }
}
